@@ -20,19 +20,21 @@ import numpy as np
 import sklearn
 from sklearn.externals.joblib import Parallel, delayed, cpu_count
 from sklearn import svm
-from sklearn.cross_validation import cross_val_score
 from sklearn.base import BaseEstimator
+from sklearn.exceptions import ConvergenceWarning
 
 from .. import masking
 from ..image.resampling import coord_transform
 from ..input_data.nifti_spheres_masker import _apply_mask_and_get_affinity
 from .._utils.compat import _basestring
+from .._utils import check_niimg_4d
+from sklearn.model_selection import cross_val_score
 
 ESTIMATOR_CATALOG = dict(svc=svm.LinearSVC, svr=svm.SVR)
 
 
-def search_light(X, y, estimator, A, scoring=None, cv=None, n_jobs=-1,
-                 verbose=0):
+def search_light(X, y, estimator, A, groups=None, scoring=None,
+                 cv=None, n_jobs=-1, verbose=0):
     """Function for computing a search_light
 
     Parameters
@@ -50,6 +52,10 @@ def search_light(X, y, estimator, A, scoring=None, cv=None, n_jobs=-1,
         adjacency matrix. Defines for each feature the neigbhoring features
         following a given structure of the data.
 
+    groups : array-like, optional
+        group label for each sample for cross validation. default None
+        NOTE: will have no effect for scikit learn < 0.18
+
     scoring : string or callable, optional
         The scoring strategy to use. See the scikit-learn documentation
         for possible values.
@@ -75,12 +81,14 @@ def search_light(X, y, estimator, A, scoring=None, cv=None, n_jobs=-1,
         search_light scores
     """
     group_iter = GroupIterator(A.shape[0], n_jobs)
-    scores = Parallel(n_jobs=n_jobs, verbose=verbose)(
-        delayed(_group_iter_search_light)(
-            A.rows[list_i],
-            estimator, X, y, scoring, cv,
-            thread_id + 1, A.shape[0], verbose)
-        for thread_id, list_i in enumerate(group_iter))
+    with warnings.catch_warnings():  # might not converge
+        warnings.simplefilter('ignore', ConvergenceWarning)
+        scores = Parallel(n_jobs=n_jobs, verbose=verbose)(
+            delayed(_group_iter_search_light)(
+                A.rows[list_i],
+                estimator, X, y, groups, scoring, cv,
+                thread_id + 1, A.shape[0], verbose)
+            for thread_id, list_i in enumerate(group_iter))
     return np.concatenate(scores)
 
 
@@ -111,7 +119,7 @@ class GroupIterator(object):
             yield list_i
 
 
-def _group_iter_search_light(list_rows, estimator, X, y,
+def _group_iter_search_light(list_rows, estimator, X, y, groups,
                              scoring, cv, thread_id, total, verbose=0):
     """Function for grouped iterations of search_light
 
@@ -130,6 +138,8 @@ def _group_iter_search_light(list_rows, estimator, X, y,
     y : array-like
         target variable to predict.
 
+    groups : array-like, optional
+        group label for each sample for cross validation.
     scoring : string or callable, optional
         Scoring strategy to use. See the scikit-learn documentation.
         If callable, takes as arguments the fitted estimator, the
@@ -158,11 +168,8 @@ def _group_iter_search_light(list_rows, estimator, X, y,
     t0 = time.time()
     for i, row in enumerate(list_rows):
         kwargs = dict()
-        if not LooseVersion(sklearn.__version__) < LooseVersion('0.15'):
-            kwargs['scoring'] = scoring
-        elif scoring is not None:
-            warnings.warn('Scikit-learn version is too old. '
-                          'scoring argument ignored', stacklevel=2)
+        kwargs['scoring'] = scoring
+        kwargs['groups'] = groups
         par_scores[i] = np.mean(cross_val_score(estimator, X[:, row],
                                                 y, cv=cv, n_jobs=1,
                                                 **kwargs))
@@ -196,11 +203,11 @@ class SearchLight(BaseEstimator):
     Parameters
     -----------
     mask_img : Niimg-like object
-        See http://nilearn.github.io/manipulating_images/input_output.html.
+        See http://nilearn.github.io/manipulating_images/input_output.html
         boolean image giving location of voxels containing usable signals.
 
     process_mask_img : Niimg-like object, optional
-        See http://nilearn.github.io/manipulating_images/input_output.html.
+        See http://nilearn.github.io/manipulating_images/input_output.html
         boolean image giving voxels on which searchlight should be
         computed.
 
@@ -260,21 +267,29 @@ class SearchLight(BaseEstimator):
         self.cv = cv
         self.verbose = verbose
 
-    def fit(self, imgs, y):
+    def fit(self, imgs, y, groups=None):
         """Fit the searchlight
 
         Parameters
         ----------
         imgs : Niimg-like object
-            See http://nilearn.github.io/manipulating_images/input_output.html.
+            See http://nilearn.github.io/manipulating_images/input_output.html
             4D image.
 
         y : 1D array-like
             Target variable to predict. Must have exactly as many elements as
             3D images in img.
 
+        groups : array-like, optional
+            group label for each sample for cross validation. Must have
+            exactly as many elements as 3D images in img. default None
+            NOTE: will have no effect for scikit learn < 0.18
+
         """
 
+        # check if image is 4D
+        imgs = check_niimg_4d(imgs)
+
         # Get the seeds
         process_mask_img = self.process_mask_img
         if self.process_mask_img is None:
@@ -291,13 +306,13 @@ class SearchLight(BaseEstimator):
 
         X, A = _apply_mask_and_get_affinity(
             process_mask_coords, imgs, self.radius, True,
-            mask_img=self.process_mask_img)
+            mask_img=self.mask_img)
 
         estimator = self.estimator
         if isinstance(estimator, _basestring):
             estimator = ESTIMATOR_CATALOG[estimator]()
 
-        scores = search_light(X, y, estimator, A,
+        scores = search_light(X, y, estimator, A, groups,
                               self.scoring, self.cv, self.n_jobs,
                               self.verbose)
         scores_3D = np.zeros(process_mask.shape)
