@@ -5,12 +5,11 @@ This is a good initialization method for ICA.
 import numpy as np
 from sklearn.externals.joblib import Memory
 from sklearn.utils.extmath import randomized_svd
-from sklearn.base import TransformerMixin
 
-from .base import BaseDecomposition, mask_and_reduce
+from .base import BaseDecomposition
 
 
-class MultiPCA(BaseDecomposition, TransformerMixin):
+class MultiPCA(BaseDecomposition):
     """Perform Multi Subject Principal Component Analysis.
 
     Perform a PCA on each subject, stack the results, and reduce them
@@ -20,7 +19,7 @@ class MultiPCA(BaseDecomposition, TransformerMixin):
     Parameters
     ----------
     n_components: int
-        Number of components to extract
+        Number of components to extract. By default n_components=20.
 
     do_cca: boolean, optional
         Indicate if a Canonical Correlation Analysis must be run after the
@@ -39,10 +38,30 @@ class MultiPCA(BaseDecomposition, TransformerMixin):
         it will be computed automatically by a MultiNiftiMasker with default
         parameters.
 
+    mask_strategy: {'background', 'epi' or 'template'}, optional
+        The strategy used to compute the mask: use 'background' if your
+        images present a clear homogeneous background, 'epi' if they
+        are raw EPI images, or you could use 'template' which will
+        extract the gray matter part of your data by resampling the MNI152
+        brain mask for your data's field of view.
+        Depending on this value, the mask will be computed from
+        masking.compute_background_mask, masking.compute_epi_mask or
+        masking.compute_gray_matter_mask. Default is 'epi'.
+
+    mask_args: dict, optional
+        If mask is None, these are additional parameters passed to
+        masking.compute_background_mask or masking.compute_epi_mask
+        to fine-tune mask computation. Please see the related documentation
+        for details.
+
     standardize : boolean, optional
         If standardize is True, the time-series are centered and normed:
         their variance is put to 1 in the time dimension.
 
+    detrend : boolean, optional
+        If detrend is True, the time-series will be detrended before
+        components extraction.
+
     target_affine: 3x3 or 4x4 matrix, optional
         This parameter is passed to image.resample_img. Please see the
         related documentation for details.
@@ -51,11 +70,11 @@ class MultiPCA(BaseDecomposition, TransformerMixin):
         This parameter is passed to image.resample_img. Please see the
         related documentation for details.
 
-    low_pass: False or float, optional
+    low_pass: None or float, optional
         This parameter is passed to signal.clean. Please see the related
         documentation for details
 
-    high_pass: False or float, optional
+    high_pass: None or float, optional
         This parameter is passed to signal.clean. Please see the related
         documentation for details
 
@@ -88,7 +107,7 @@ class MultiPCA(BaseDecomposition, TransformerMixin):
         of `mask` and other NiftiMasker related parameters as initialization.
 
     `mask_img_` : Niimg-like object
-        See http://nilearn.github.io/manipulating_images/input_output.html.
+        See http://nilearn.github.io/manipulating_images/input_output.html
         The mask of the data. If no mask was given at masker creation, contains
         the automatically computed mask.
 
@@ -96,6 +115,17 @@ class MultiPCA(BaseDecomposition, TransformerMixin):
         Array of masked extracted components. They can be unmasked thanks to
         the `masker_` attribute.
 
+        Deprecated since version 0.4.1. Use `components_img_` instead.
+
+    `components_img_` : 4D Nifti image
+        4D image giving the extracted PCA components. Each 3D image is a
+        component.
+
+        New in version 0.4.1.
+
+    `variance_` : numpy array (n_components,)
+        The amount of variance explained by each of the selected components.
+
     """
 
     def __init__(self, n_components=20,
@@ -131,44 +161,21 @@ class MultiPCA(BaseDecomposition, TransformerMixin):
                                    n_jobs=n_jobs,
                                    verbose=verbose)
 
-    def fit(self, imgs, y=None, confounds=None):
-        """Compute the mask and the components
-
-        Parameters
-        ----------
-        imgs: list of Niimg-like objects
-            See http://nilearn.github.io/manipulating_images/input_output.html.
-            Data on which the PCA must be calculated. If this is a list,
-            the affine is considered the same for all.
-
-        confounds: CSV file path or 2D matrix
-            This parameter is passed to nilearn.signal.clean. Please see the
-            related documentation for details
-
-        """
-        BaseDecomposition.fit(self, imgs)
-
-        data = mask_and_reduce(self.masker_, imgs,
-                               confounds=confounds,
-                               n_components=self.n_components,
-                               random_state=self.random_state,
-                               memory=self.memory,
-                               memory_level=max(0, self.memory_level - 1),
-                               n_jobs=self.n_jobs)
-        self._raw_fit(data)
-        return self
-
     def _raw_fit(self, data):
         """Helper function that directly process unmasked data"""
         if self.do_cca:
             S = np.sqrt(np.sum(data ** 2, axis=1))
             S[S == 0] = 1
             data /= S[:, np.newaxis]
-        self.components_, self.variance_, _ = self._cache(
+        components_, self.variance_, _ = self._cache(
             randomized_svd, func_memory_level=2)(
             data.T, n_components=self.n_components,
             transpose=True,
             random_state=self.random_state, n_iter=3)
         if self.do_cca:
             data *= S[:, np.newaxis]
-        self.components_ = self.components_.T
+        self.components_ = components_.T
+        if hasattr(self, "masker_"):
+            self.components_img_ = self.masker_.inverse_transform(
+                components_.T)
+        return components_
