@@ -11,14 +11,11 @@ import distutils.version
 import warnings
 
 import numpy as np
-import scipy
-from scipy import signal, stats, linalg
+from scipy import stats, linalg, signal as sp_signal
 from sklearn.utils import gen_even_slices, as_float_array
-from distutils.version import LooseVersion
 
 from ._utils.compat import _basestring
-from ._utils.numpy_conversions import csv_to_array
-from ._utils import check_niimg_4d
+from ._utils.numpy_conversions import csv_to_array, as_ndarray
 
 NP_VERSION = distutils.version.LooseVersion(np.version.short_version).version
 
@@ -168,18 +165,32 @@ def _detrend(signals, inplace=False, type="linear", n_batches=10):
 
 def _check_wn(btype, freq, nyq):
     wn = freq / float(nyq)
-    if wn > 1.:
+    if wn >= 1.:
+        # results looked unstable when the critical frequencies are
+        # exactly at the Nyquist frequency. See issue at SciPy
+        # https://github.com/scipy/scipy/issues/6265. Before, SciPy 1.0.0 ("wn
+        # should be btw 0 and 1"). But, after ("0 < wn < 1"). Due to unstable
+        # results as pointed in the issue above. Hence, we forced the
+        # critical frequencies to be slightly less than 1. but not 1.
+        wn = 1 - 10 * np.finfo(1.).eps
         warnings.warn(
             'The frequency specified for the %s pass filter is '
             'too high to be handled by a digital filter (superior to '
             'nyquist frequency). It has been lowered to %.2f (nyquist '
-            'frequency).' % (btype, nyq))
-        wn = 1.
+            'frequency).' % (btype, wn))
+
+    if wn < 0.0: # equal to 0.0 is okay
+        wn = np.finfo(1.).eps
+        warnings.warn(
+            'The frequency specified for the %s pass filter is '
+            'too low to be handled by a digital filter (must be non-negative).'
+            ' It has been set to eps: %.5e' % (btype, wn))
+
     return wn
 
 
 def butterworth(signals, sampling_rate, low_pass=None, high_pass=None,
-                order=5, copy=False, save_memory=False):
+                order=5, copy=False):
     """ Apply a low-pass, high-pass or band-pass Butterworth filter
 
     Apply a filter to remove signal below the `low` frequency and above the
@@ -219,9 +230,9 @@ def butterworth(signals, sampling_rate, low_pass=None, high_pass=None,
     """
     if low_pass is None and high_pass is None:
         if copy:
-            return signal.copy()
+            return signals.copy()
         else:
-            return signal
+            return signals
 
     if low_pass is not None and high_pass is not None \
             and high_pass >= low_pass:
@@ -247,29 +258,26 @@ def butterworth(signals, sampling_rate, low_pass=None, high_pass=None,
     else:
         critical_freq = critical_freq[0]
 
-    b, a = signal.butter(order, critical_freq, btype=btype)
+    b, a = sp_signal.butter(order, critical_freq, btype=btype, output='ba')
     if signals.ndim == 1:
         # 1D case
-        output = signal.filtfilt(b, a, signals)
+        output = sp_signal.filtfilt(b, a, signals)
         if copy:  # filtfilt does a copy in all cases.
             signals = output
         else:
             signals[...] = output
     else:
         if copy:
-            if (LooseVersion(scipy.__version__) < LooseVersion('0.10.0')):
-                # filtfilt is 1D only in scipy 0.9.0
-                signals = signals.copy()
-                for timeseries in signals.T:
-                    timeseries[:] = signal.filtfilt(b, a, timeseries)
-            else:
-                # No way to save memory when a copy has been requested,
-                # because filtfilt does out-of-place processing
-                signals = signal.filtfilt(b, a, signals, axis=0)
+            # No way to save memory when a copy has been requested,
+            # because filtfilt does out-of-place processing
+            signals = sp_signal.filtfilt(b, a, signals, axis=0)
         else:
             # Lesser memory consumption, slower.
             for timeseries in signals.T:
-                timeseries[:] = signal.filtfilt(b, a, timeseries)
+                timeseries[:] = sp_signal.filtfilt(b, a, timeseries)
+
+            # results returned in-place
+
     return signals
 
 
@@ -347,16 +355,17 @@ def _ensure_float(data):
 
 
 def clean(signals, sessions=None, detrend=True, standardize=True,
-          confounds=None, low_pass=None, high_pass=None, t_r=2.5):
+          confounds=None, low_pass=None, high_pass=None, t_r=None,
+          ensure_finite=False):
     """Improve SNR on masked fMRI signals.
 
     This function can do several things on the input signals, in
     the following order:
 
     - detrend
-    - standardize
-    - remove confounds
     - low- and high-pass filter
+    - remove confounds
+    - standardize
 
     Low-pass filtering improves specificity.
 
@@ -365,6 +374,10 @@ def clean(signals, sessions=None, detrend=True, standardize=True,
 
     Filtering is only meaningful on evenly-sampled signals.
 
+    According to Lindquist et al. (2018), removal of confounds will be done
+    orthogonally to temporal filters (low- and/or high-pass filters), if both
+    are specified.
+
     Parameters
     ----------
     signals: numpy.ndarray
@@ -386,7 +399,8 @@ def clean(signals, sessions=None, detrend=True, standardize=True,
         signal, as if all were in the same array.
 
     t_r: float
-        Repetition time, in second (sampling period).
+        Repetition time, in second (sampling period). Set to None if not
+        specified. Mandatory if used together with low_pass or high_pass.
 
     low_pass, high_pass: float
         Respectively low and high cutoff frequencies, in Hertz.
@@ -398,6 +412,10 @@ def clean(signals, sessions=None, detrend=True, standardize=True,
     standardize: bool
         If True, returned signals are set to unit variance.
 
+    ensure_finite: bool
+        If True, the non-finite values (NANs and infs) found in the data
+        will be replaced by zeros.
+
     Returns
     -------
     cleaned_signals: numpy.ndarray
@@ -412,16 +430,38 @@ def clean(signals, sessions=None, detrend=True, standardize=True,
     Linear Approach". Human Brain Mapping 2, no 4 (1994): 189-210.
     <http://dx.doi.org/10.1002/hbm.460020402>`_
 
+    Orthogonalization between temporal filters and confound removal is based on
+    suggestions in `Lindquist, M., Geuter, S., Wager, T., & Caffo, B. (2018).
+    Modular preprocessing pipelines can reintroduce artifacts into fMRI data.
+    bioRxiv, 407676. <http://dx.doi.org/10.1101/407676>`_
+
     See Also
     --------
         nilearn.image.clean_img
     """
 
+    if isinstance(low_pass, bool):
+        raise TypeError("low pass must be float or None but you provided "
+                        "low_pass='{0}'".format(low_pass))
+    if isinstance(high_pass, bool):
+        raise TypeError("high pass must be float or None but you provided "
+                        "high_pass='{0}'".format(high_pass))
+
     if not isinstance(confounds,
                       (list, tuple, _basestring, np.ndarray, type(None))):
         raise TypeError("confounds keyword has an unhandled type: %s"
                         % confounds.__class__)
 
+    if not isinstance(ensure_finite, bool):
+        raise ValueError("'ensure_finite' must be boolean type True or False "
+                         "but you provided ensure_finite={0}".format(ensure_finite))
+
+    if not isinstance(signals, np.ndarray):
+        signals = as_ndarray(signals)
+
+    if ensure_finite:
+        signals[np.logical_not(np.isfinite(signals))] = 0
+
     # Read confounds
     if confounds is not None:
         if not isinstance(confounds, (list, tuple)):
@@ -472,17 +512,35 @@ def clean(signals, sessions=None, detrend=True, standardize=True,
                 clean(signals[sessions == s],
                       detrend=detrend, standardize=standardize,
                       confounds=session_confounds, low_pass=low_pass,
-                      high_pass=high_pass, t_r=2.5)
+                      high_pass=high_pass, t_r=t_r)
 
     # detrend
     signals = _ensure_float(signals)
     signals = _standardize(signals, normalize=False, detrend=detrend)
 
+    # Apply low- and high-pass filters
+    if low_pass is not None or high_pass is not None:
+        if t_r is None:
+            raise ValueError("Repetition time (t_r) must be specified for "
+                             "filtering. You specified None.")
+
+        signals = butterworth(signals, sampling_rate=1. / t_r,
+                              low_pass=low_pass, high_pass=high_pass)
+
     # Remove confounds
     if confounds is not None:
         confounds = _ensure_float(confounds)
+
+        # Apply low- and high-pass filters to keep filters orthogonal
+        # (according to Lindquist et al. (2018))
+        if low_pass is not None or high_pass is not None:
+
+            confounds = butterworth(confounds, sampling_rate=1. / t_r,
+                                    low_pass=low_pass, high_pass=high_pass)
+
         confounds = _standardize(confounds, normalize=standardize,
                                  detrend=detrend)
+
         if not standardize:
             # Improve numerical stability by controlling the range of
             # confounds. We don't rely on _standardize as it removes any
@@ -491,30 +549,13 @@ def clean(signals, sessions=None, detrend=True, standardize=True,
             confound_max[confound_max == 0] = 1
             confounds /= confound_max
 
-        if (LooseVersion(scipy.__version__) > LooseVersion('0.9.0')):
-            # Pivoting in qr decomposition was added in scipy 0.10
-            Q, R, _ = linalg.qr(confounds, mode='economic', pivoting=True)
-            Q = Q[:, np.abs(np.diag(R)) > np.finfo(np.float).eps * 100.]
-            signals -= Q.dot(Q.T).dot(signals)
-        else:
-            Q, R = linalg.qr(confounds, mode='economic')
-            non_null_diag = np.abs(np.diag(R)) > np.finfo(np.float).eps * 100.
-            if np.all(non_null_diag):
-                signals -= Q.dot(Q.T).dot(signals)
-            elif np.any(non_null_diag):
-                R = R[:, non_null_diag]
-                confounds = confounds[:, non_null_diag]
-                inv = scipy.linalg.inv(np.dot(R.T, R))
-                signals -= confounds.dot(inv).dot(confounds.T).dot(signals)
-
-    if low_pass is not None or high_pass is not None:
-        signals = butterworth(signals, sampling_rate=1. / t_r,
-                              low_pass=low_pass, high_pass=high_pass)
+        # Pivoting in qr decomposition was added in scipy 0.10
+        Q, R, _ = linalg.qr(confounds, mode='economic', pivoting=True)
+        Q = Q[:, np.abs(np.diag(R)) > np.finfo(np.float).eps * 100.]
+        signals -= Q.dot(Q.T).dot(signals)
 
     if standardize:
         signals = _standardize(signals, normalize=True, detrend=False)
         signals *= np.sqrt(signals.shape[0])  # for unit variance
 
     return signals
-
-
